==> http-1.2.tar.gz: http-1.2/base64.c <==
#include <stddef.h>

#include "base64.h"

static unsigned char base64[64] = 
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz"
        "0123456789+/";

void base64_encode(
    void *dest,
    size_t destlen,
    void *src,
    size_t srclen,
    size_t linelen
){
    /*
     * base64_encode - convert binary data to base64
     *
     * Writes a base64 encoding of |srclen| bytes starting at |src|
     * into |dest|. No more than |destlen| bytes will be written.
     * After every |linelen| bytes written, a CRLF sequence will be
     * inserted. 
     *
     * The base64 encoding will be in the run time character set, not
     * US-ASCII. This is probably correct for e-mail (where a character
     * set conversion will occur at a gateway), but not for HTTP.
     * Most sites won't care of course, because their run time character
     * set is some some superset of US-ASCII.
     *
     * MIME (RFC 2045) mandates a maximum line length of 76 characters.
     * HTTP basic-cookies have arbirtrary line length.
     */
    unsigned char *s = src;
    unsigned char *se = s + srclen;
    unsigned char *d = dest;
    unsigned char *de = d + destlen;
    size_t cc = 0;
    unsigned int sr;
    linelen &= ~3;
    while (s < se && d < de) {

        if (cc >= linelen) {
            if(d > de - 2) break;
            *d++ = '\r';
            *d++ = '\n';
            cc = 0;
        }
        if(d > de - 4) break;
        sr = *s++;
        *d++ = base64[sr >> 2];
        sr = (sr & 3) << 8;
        if (s < se) {
            sr |= *s++;
            *d++ = base64[sr >> 4];
            sr = (sr & 0xF) << 8;
            if (s < se) {
                sr |= *s++;
                *d++ = base64[sr >> 6];
                *d++ = base64[sr & 0x3F];
            } else {
                *d++ = base64[sr >> 6];
                *d++ = '=';
            }
        } else {
            *d++ = base64[sr >> 4];
            *d++ = '=';
            *d++ = '=';
        }
        cc += 4;
    }
}

#ifdef TEST
#include <stdio.h>
#include <string.h>

char buffer[1024];
struct {
    void *dest;
    size_t destlen;
    void *src;
    size_t srclen;
    size_t linelen;
    void *check;
} examples[] = {
    { buffer, 1024, "Aladdin:open sesame", 19, -1, "QWxhZGRpbjpvcGVuIHNlc2FtZQ==" },
};


int main(int argc, char **argv) {
    int i;
    int rc = 0;
    for (i = 0; i < sizeof(examples)/ sizeof(examples[0]); i++) {
        printf ("Example %d ", i);
        memset(examples[i].dest, 0, strlen(examples[i].check) + 1);
        base64_encode(
            examples[i].dest,
            examples[i].destlen,
            examples[i].src,
            examples[i].srclen,
            examples[i].linelen);
        if (strcmp(examples[i].dest, examples[i].check) == 0) {
            printf ("ok\n");
        } else {
            printf ("failed\n");
            printf ("\t%s\n", (char *)examples[i].dest);
            printf ("\t%s\n", (char *)examples[i].check);
            rc = 1;
        }
    }
    return rc;
}
#endif

==> http-1.2.1.tar.gz: http-1.2.1/base64.c <==
char base64_c_rcsid[] = 
    "$Id: base64.c,v 1.3 1998/01/14 18:01:11 hjp Exp $";
/* $Log: base64.c,v $
 * Revision 1.3  1998/01/14 18:01:11  hjp
 * Inserted RCS Id and changelog.
 * Fixed comments.
 * Removed real password from examples.
 *
 */

#include <stddef.h>

#include "base64.h"

static unsigned char base64[64] = 
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz"
        "0123456789+/";

size_t base64_encode(
    void *dest,
    size_t destlen,
    void *src,
    size_t srclen,
    size_t linelen
){
    /*
     * base64_encode - convert binary data to base64
     *
     * Writes a base64 encoding of |srclen| bytes starting at |src|
     * into |dest|. No more than |destlen| bytes will be written.
     * After every |linelen| bytes written, a CRLF sequence will be
     * inserted. The resulting string is NOT null-terminated.
     *
     * Returns the number of bytes written.
     *
     * The base64 encoding will be in the run time character set, not
     * US-ASCII. This is probably correct for e-mail (where a character
     * set conversion will occur at a gateway), but not for HTTP.
     * Most sites won't care of course, because their run time character
     * set is some superset of US-ASCII.
     *
     * MIME (RFC 2045) mandates a maximum line length of 76 characters.
     * HTTP basic-cookies have arbirtrary line length.
     */
    unsigned char *s = src;
    unsigned char *se = s + srclen;
    unsigned char *d = dest;
    unsigned char *de = d + destlen;
    size_t cc = 0;
    unsigned int sr;
    linelen &= ~3;
    while (s < se && d < de) {

        if (cc >= linelen) {
            if(d > de - 2) break;
            *d++ = '\r';
            *d++ = '\n';
            cc = 0;
        }
        if(d > de - 4) break;
        sr = *s++;
        *d++ = base64[sr >> 2];
        sr = (sr & 3) << 8;
        if (s < se) {
            sr |= *s++;
            *d++ = base64[sr >> 4];
            sr = (sr & 0xF) << 8;
            if (s < se) {
                sr |= *s++;
                *d++ = base64[sr >> 6];
                *d++ = base64[sr & 0x3F];
            } else {
                *d++ = base64[sr >> 6];
                *d++ = '=';
            }
        } else {
            *d++ = base64[sr >> 4];
            *d++ = '=';
            *d++ = '=';
        }
        cc += 4;
    }
    return d - (unsigned char *)dest;
}

#ifdef TEST
#include <stdio.h>
#include <string.h>

char buffer[1024];
struct {
    void *dest;
    size_t destlen;
    void *src;
    size_t srclen;
    size_t linelen;
    void *check;
} examples[] = {
    { buffer, 1024, "Aladdin:open sesame", 19, -1, "QWxhZGRpbjpvcGVuIHNlc2FtZQ==" },
};


int main(int argc, char **argv) {
    int i;
    int rc = 0;
    for (i = 0; i < sizeof(examples)/ sizeof(examples[0]); i++) {
        printf ("Example %d ", i);
        memset(examples[i].dest, 0, strlen(examples[i].check) + 1);
        base64_encode(
            examples[i].dest,
            examples[i].destlen,
            examples[i].src,
            examples[i].srclen,
            examples[i].linelen);
        if (strcmp(examples[i].dest, examples[i].check) == 0) {
            printf ("ok\n");
        } else {
            printf ("failed\n");
            printf ("\t%s\n", (char *)examples[i].dest);
            printf ("\t%s\n", (char *)examples[i].check);
            rc = 1;
        }
    }
    return rc;
}
#endif
